@@ -84,6 +84,9 @@ bool icsneo_serialNumToString(uint32_t num, char* str, size_t* count) {
 }
 
 uint32_t icsneo_serialStringToNum(const char* str) {
+	if(!str)
+		return 0;
+
 	return Device::SerialStringToNum(str);
 }
 
@@ -141,8 +144,17 @@ bool icsneo_closeDevice(const neodevice_t* device) {
 		if((*it).get() == device->device)
 			itemsToDelete.push_back(it);
 	}
-	for(auto it : itemsToDelete)
+	for(auto it : itemsToDelete) {
+		// Move it back into connectable devices so we can open it again.
+		// Without this we will be unable to use/reopen the device due to 
+		// icsneo_isValidNeoDevice / icsneo_openDevice checks against this
+		// container. Since its closed we are in a connectable state again.
+		// Notice: When we search again this will be cleaned up by 
+		// icsneo_freeUnconnectedDevices()
+		connectableFoundDevices.push_back(*it);
+		// Remove it from the connected devices as we are no longer connected.
 		connectedDevices.erase(it);
+	}
 
 	return true;
 }
@@ -248,7 +260,7 @@ int icsneo_addMessageCallback(const neodevice_t* device, void (*callback)(neomes
 		return -1;
 
 	return device->device->addMessageCallback(
-		MessageCallback(
+		std::make_shared<MessageCallback>(
 			[=](std::shared_ptr<icsneo::Message> msg) {
 				return callback(CreateNeoMessage(msg));
 			}
@@ -262,6 +274,12 @@ bool icsneo_removeMessageCallback(const neodevice_t* device, int id) {
 	return device->device->removeMessageCallback(id);
 }
 
+neonetid_t icsneo_getNetworkByNumber(const neodevice_t* device, neonettype_t type, unsigned int number) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+	return neonetid_t(device->device->getNetworkByNumber(icsneo::Network::Type(type), size_t(number)).getNetID());
+}
+
 bool icsneo_getProductName(const neodevice_t* device, char* str, size_t* maxLength) {
 	// TAG String copy function
 	if(maxLength == nullptr) {
@@ -272,7 +290,7 @@ bool icsneo_getProductName(const neodevice_t* device, char* str, size_t* maxLeng
 	if(!icsneo_isValidNeoDevice(device))
 		return false;
 
-	std::string output = device->device->getType().toString();
+	std::string output = device->device->getProductName();
 
 	if(str == nullptr) {
 		*maxLength = output.length();
@@ -295,7 +313,7 @@ bool icsneo_getProductNameForType(devicetype_t type, char* str, size_t* maxLengt
 		return false;
 	}
 
-	std::string output = DeviceType(type).toString();
+	std::string output = DeviceType(type).getGenericProductName();
 
 	if(str == nullptr) {
 		*maxLength = output.length();
@@ -412,28 +430,28 @@ bool icsneo_settingsApplyStructureTemporary(const neodevice_t* device, const voi
 	return icsneo_settingsWriteStructure(device, structure, structureSize) && icsneo_settingsApplyTemporary(device);
 }
 
-int64_t icsneo_getBaudrate(const neodevice_t* device, uint16_t netid) {
+int64_t icsneo_getBaudrate(const neodevice_t* device, neonetid_t netid) {
 	if(!icsneo_isValidNeoDevice(device))
 		return -1;
 
 	return device->device->settings->getBaudrateFor(netid);
 }
 
-bool icsneo_setBaudrate(const neodevice_t* device, uint16_t netid, int64_t newBaudrate) {
+bool icsneo_setBaudrate(const neodevice_t* device, neonetid_t netid, int64_t newBaudrate) {
 	if(!icsneo_isValidNeoDevice(device))
 		return false;
 
 	return device->device->settings->setBaudrateFor(netid, newBaudrate);
 }
 
-int64_t icsneo_getFDBaudrate(const neodevice_t* device, uint16_t netid) {
+int64_t icsneo_getFDBaudrate(const neodevice_t* device, neonetid_t netid) {
 	if(!icsneo_isValidNeoDevice(device))
 		return -1;
 
 	return device->device->settings->getFDBaudrateFor(netid);
 }
 
-bool icsneo_setFDBaudrate(const neodevice_t* device, uint16_t netid, int64_t newBaudrate) {
+bool icsneo_setFDBaudrate(const neodevice_t* device, neonetid_t netid, int64_t newBaudrate) {
 	if(!icsneo_isValidNeoDevice(device))
 		return false;
 
@@ -444,7 +462,10 @@ bool icsneo_transmit(const neodevice_t* device, const neomessage_t* message) {
 	if(!icsneo_isValidNeoDevice(device))
 		return false;
 
-	return device->device->transmit(CreateMessageFromNeoMessage(message));
+	if(auto frame = std::dynamic_pointer_cast<icsneo::Frame>(CreateMessageFromNeoMessage(message)))
+		return device->device->transmit(frame);
+
+	return false;
 }
 
 bool icsneo_transmitMessages(const neodevice_t* device, const neomessage_t* messages, size_t count) {
@@ -474,6 +495,11 @@ bool icsneo_describeDevice(const neodevice_t* device, char* str, size_t* maxLeng
 	if(!icsneo_isValidNeoDevice(device))
 		return false;
 
+	if(!str) {
+		*maxLength = device->device->describe().length();
+		return false;
+	}
+
 	std::string output = device->device->describe();
 
 	*maxLength = output.copy(str, *maxLength);
@@ -608,7 +634,7 @@ bool icsneo_getSupportedDevices(devicetype_t* devices, size_t* count) {
 	return true;
 }
 
-extern bool DLLExport icsneo_getTimestampResolution(const neodevice_t* device, uint16_t* resolution) {
+bool icsneo_getTimestampResolution(const neodevice_t* device, uint16_t* resolution) {
 	if(!icsneo_isValidNeoDevice(device))
 		return false;
 
@@ -620,3 +646,144 @@ extern bool DLLExport icsneo_getTimestampResolution(const neodevice_t* device, u
 	*resolution = device->device->getTimestampResolution();
 	return true;
 }
+
+bool icsneo_getDigitalIO(const neodevice_t* device, neoio_t type, uint32_t number, bool* value) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	if(value == nullptr) {
+		EventManager::GetInstance().add(APIEvent::Type::RequiredParameterNull, APIEvent::Severity::Error);
+		return false;
+	}
+
+	const std::optional<bool> val = device->device->getDigitalIO(static_cast<icsneo::IO>(type), number);
+	if(!val.has_value())
+		return false;
+
+	*value = *val;
+	return true;
+}
+
+bool icsneo_setDigitalIO(const neodevice_t* device, neoio_t type, uint32_t number, bool value) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	return device->device->setDigitalIO(static_cast<icsneo::IO>(type), number, value);
+}
+
+bool icsneo_isTerminationSupportedFor(const neodevice_t* device, neonetid_t netid) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	return device->device->settings->isTerminationSupportedFor(Network(netid));
+}
+
+bool icsneo_canTerminationBeEnabledFor(const neodevice_t* device, neonetid_t netid) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	return device->device->settings->canTerminationBeEnabledFor(Network(netid));
+}
+
+bool icsneo_isTerminationEnabledFor(const neodevice_t* device, neonetid_t netid) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	return device->device->settings->isTerminationEnabledFor(Network(netid)).value_or(false);
+}
+
+bool icsneo_setTerminationFor(const neodevice_t* device, neonetid_t netid, bool enabled) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	return device->device->settings->setTerminationFor(Network(netid), enabled);
+}
+
+bool icsneo_getRTC(const neodevice_t* device, uint64_t* output)
+{
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	const std::optional<std::chrono::time_point<std::chrono::system_clock>> rtc = device->device->getRTC();
+	if(!rtc)
+		return false;
+
+	auto duration = std::chrono::duration_cast<std::chrono::seconds>(rtc->time_since_epoch());
+	*output = static_cast<uint64_t>(duration.count());
+
+	return true;
+}
+
+bool icsneo_setRTC(const neodevice_t* device, uint64_t input)
+{
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	std::chrono::seconds duration(input);
+	const std::chrono::system_clock::time_point time(duration);
+	return device->device->setRTC(time);
+}
+
+int icsneo_getDeviceStatus(const neodevice_t* device, void* status, size_t* size) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	if(status == nullptr || size == nullptr)
+		return false;
+	
+	std::shared_ptr<Message> msg = device->device->com->waitForMessageSync([&]() {
+		return device->device->com->sendCommand(Command::RequestStatusUpdate);
+	}, std::make_shared<MessageFilter>(Network::NetID::DeviceStatus), std::chrono::milliseconds(100));
+
+	if(!msg) // Did not receive a message
+		return false;
+	
+	auto rawMessage = std::static_pointer_cast<RawMessage>(msg);
+	if(!rawMessage || (rawMessage->network.getNetID() != Network::NetID::DeviceStatus))
+		return false;
+
+	if(*size < rawMessage->data.size())
+		return false;
+	
+	std::copy(rawMessage->data.begin(), rawMessage->data.end(), static_cast<uint8_t*>(status));
+	*size = rawMessage->data.size();
+
+	return true;
+}
+
+bool icsneo_isOnlineSupported(const neodevice_t* device) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+	
+	return device->device->isOnlineSupported();
+}
+
+bool icsneo_requestTC10Wake(const neodevice_t* device, neonetid_t netid) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	return device->device->requestTC10Wake((Network::NetID)netid);
+}
+
+bool icsneo_requestTC10Sleep(const neodevice_t* device, neonetid_t netid) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+
+	return device->device->requestTC10Sleep((Network::NetID)netid);
+}
+
+bool icsneo_getTC10Status(const neodevice_t* device, neonetid_t netid, neotc10status_t* status) {
+	if(!icsneo_isValidNeoDevice(device))
+		return false;
+	
+	const auto statusMsg = device->device->getTC10Status((Network::NetID)netid);
+
+	if(!statusMsg)
+		return false;
+
+	status->wakeStatus = (neotc10wakestatus_t)statusMsg->wakeStatus;
+	status->sleepStatus = (neotc10sleepstatus_t)statusMsg->sleepStatus;
+
+	return true;
+}
+
