@@ -14,15 +14,15 @@
 using namespace icsneo;
 
 extern "C" {
-extern int DLLExport icsneoValidateHObject(void* hObject);
-extern int DLLExport icsneoWaitForRxMessagesWithTimeOut(void* hObject, unsigned int iTimeOut);
+extern int LegacyDLLExport icsneoValidateHObject(void* hObject);
+extern int LegacyDLLExport icsneoWaitForRxMessagesWithTimeOut(void* hObject, unsigned int iTimeOut);
 }
 
-int icsneoWaitForRxMessagesWithTimeOut(void* hObject, unsigned int iTimeOut) {
+int LegacyDLLExport icsneoWaitForRxMessagesWithTimeOut(void* hObject, unsigned int iTimeOut) {
 	if(!icsneoValidateHObject(hObject))
 		return false;
 	neodevice_t* device = (neodevice_t*)hObject;
 	if(device->device->getCurrentMessageCount() != 0)
 		return true;
-	return bool(device->device->com->waitForMessageSync(MessageFilter(), std::chrono::milliseconds(iTimeOut)));
+	return bool(device->device->com->waitForMessageSync({}, std::chrono::milliseconds(iTimeOut)));
 }
\ No newline at end of file
