@@ -42,6 +42,8 @@ void printMainMenu() {
 	std::cout << "I - Set HS CAN to 250K" << std::endl;
 	std::cout << "J - Set LSFT CAN to 250K" << std::endl;
 	std::cout << "K - Add/Remove a message callback" << std::endl;
+	std::cout << "L - Set Digital IO" << std::endl;
+	std::cout << "M - Set HS CAN termination" << std::endl;
 	std::cout << "X - Exit" << std::endl;
 }
 
@@ -128,7 +130,7 @@ void printDeviceWarnings(std::shared_ptr<icsneo::Device> device) {
  * \returns the entered character
  *
  * This function repeatedly prompts the user for input until a matching input is entered
- * Example usage: 
+ * Example usage:
  * char input = getCharInput(std::vector<char> {'F', 'u', 'b', 'a', 'r'});
  */
 char getCharInput(std::vector<char> allowed) {
@@ -180,6 +182,102 @@ std::shared_ptr<icsneo::Device> selectDevice(const std::vector<std::shared_ptr<i
 	return from.at(selectedDeviceNum - 1);
 }
 
+void printMessage(const std::shared_ptr<icsneo::Message>& message) {
+	switch(message->type) {
+		case icsneo::Message::Type::Frame: {
+			// A message of type Frame is guaranteed to be a Frame, so we can static cast safely
+			auto frame = std::static_pointer_cast<icsneo::Frame>(message);
+			switch(frame->network.getType()) {
+				case icsneo::Network::Type::CAN: {
+					// A message of type CAN is guaranteed to be a CANMessage, so we can static cast safely
+					auto canMessage = std::static_pointer_cast<icsneo::CANMessage>(message);
+
+					std::cout << "\t\t" << frame->network << ' ';
+					if(canMessage->isCANFD) {
+						std::cout << "FD ";
+						if(!canMessage->baudrateSwitch)
+							std::cout << "(No BRS) ";
+					}
+
+					// Print the Arbitration ID
+					std::cout << "0x" << std::hex << std::setw(canMessage->isExtended ? 8 : 3)
+						<< std::setfill('0') << canMessage->arbid;
+
+					// Print the DLC
+					std::cout << std::dec << " [" << canMessage->data.size() << "] ";
+
+					// Print the data
+					for(auto& databyte : canMessage->data)
+						std::cout << std::hex << std::setw(2) << (uint32_t)databyte << ' ';
+
+					// Print the timestamp
+					std::cout << std::dec << '(' << canMessage->timestamp << " ns since 1/1/2007)\n";
+					break;
+				}
+				case icsneo::Network::Type::Ethernet: {
+					auto ethMessage = std::static_pointer_cast<icsneo::EthernetMessage>(message);
+
+					std::cout << "\t\t" << ethMessage->network << " Frame - " << std::dec
+						<< ethMessage->data.size() << " bytes on wire\n";
+					std::cout << "\t\t  Timestamped:\t"<< ethMessage->timestamp << " ns since 1/1/2007\n";
+
+					// The MACAddress may be printed directly or accessed with the `data` member
+					std::cout << "\t\t  Source:\t" << ethMessage->getSourceMAC() << "\n";
+					std::cout << "\t\t  Destination:\t" << ethMessage->getDestinationMAC();
+
+					// Print the data
+					for(size_t i = 0; i < ethMessage->data.size(); i++) {
+						if(i % 8 == 0)
+							std::cout << "\n\t\t  " << std::hex << std::setw(4) << std::setfill('0') << i << '\t';
+						std::cout << std::hex << std::setw(2) << (uint32_t)ethMessage->data[i] << ' ';
+					}
+
+					std::cout << std::dec << std::endl;
+					break;
+				}
+				case icsneo::Network::Type::ISO9141: {
+					// Note that the default settings on some devices have ISO9141 disabled by default in favor of LIN
+					// and that this example loads the device defaults at the very end.
+					// A message of type ISO9414 is guaranteed to be an ISO9141Message, so we can static cast safely
+					auto isoMessage = std::static_pointer_cast<icsneo::ISO9141Message>(message);
+
+					std::cout << "\t\t" << isoMessage->network << ' ';
+
+					// Print the header bytes
+					std::cout << '(' << std::hex << std::setfill('0') << std::setw(2) << (uint32_t)isoMessage->header[0] << ' ';
+					std::cout << std::setfill('0') << std::setw(2) << (uint32_t)isoMessage->header[1] << ' ';
+					std::cout << std::setfill('0') << std::setw(2) << (uint32_t)isoMessage->header[2] << ") ";
+
+					// Print the data length
+					std::cout << std::dec << " [" << isoMessage->data.size() << "] ";
+
+					// Print the data
+					for(auto& databyte : isoMessage->data)
+						std::cout << std::hex << std::setfill('0') << std::setw(2) << (uint32_t)databyte << ' ';
+
+					// Print the timestamp
+					std::cout << std::dec << '(' << isoMessage->timestamp << " ns since 1/1/2007)\n";
+					break;
+				}
+				default:
+					// Ignoring non-network messages
+					break;
+			}
+			break;
+		} // end of icsneo::Message::Type::Frame
+		case icsneo::Message::Type::CANErrorCount: {
+			// A message of type CANErrorCount is guaranteed to be a CANErrorCount, so we can static cast safely
+			auto cec = std::static_pointer_cast<icsneo::CANErrorMessage>(message);
+			std::cout << "\t\t" << cec->network << " error counts changed, REC=" << cec->receiveErrorCount
+				<< " TEC=" << cec->transmitErrorCount << " (" << (cec->busOff ? "" : "Not ") << "Bus Off)" << std::endl;
+			break;
+		}
+		default:
+			// Ignoring other types of messages
+			break;
+	}
+}
+
 int main() {
 	std::cout << "Running libicsneo " << icsneo::GetVersion() << std::endl << std::endl;
 
@@ -191,7 +289,7 @@ int main() {
 	while(true) {
 		printMainMenu();
 		std::cout << std::endl;
-		char input = getCharInput(std::vector<char> {'A', 'a', 'B', 'b', 'C', 'c', 'D', 'd', 'E', 'e', 'F', 'f', 'G', 'g', 'H', 'h', 'I', 'i', 'J', 'j', 'K', 'k', 'X', 'x'});
+		char input = getCharInput(std::vector<char> {'A', 'a', 'B', 'b', 'C', 'c', 'D', 'd', 'E', 'e', 'F', 'f', 'G', 'g', 'H', 'h', 'I', 'i', 'J', 'j', 'K', 'k', 'L', 'l', 'M', 'm', 'X', 'x'});
 		std::cout << std::endl;
 
 		switch(input) {
@@ -235,7 +333,7 @@ int main() {
 			std::cout << "[1] Open" << std::endl << "[2] Close" << std::endl <<  "[3] Cancel" << std::endl << std::endl;
 			char selection = getCharInput(std::vector<char> {'1', '2', '3'});
 			std::cout << std::endl;
-			
+
 			switch(selection) {
 			case '1':
 				if(selectedDevice->open()) {
@@ -245,7 +343,7 @@ int main() {
 					std::cout << icsneo::GetLastError() << std::endl;
 					std::cout << std::endl;
 				}
-				
+
 				break;
 			case '2':
 				// Attempt to close the device
@@ -257,7 +355,7 @@ int main() {
 					std::cout << icsneo::GetLastError() << std::endl;;
 					std::cout << std::endl;
 				}
-				
+
 				break;
 			default:
 				std::cout << "Canceling!" << std::endl << std::endl;
@@ -358,7 +456,7 @@ int main() {
 			default:
 				std::cout << "Canceling!" << std::endl << std::endl;
 				break;
-			}			
+			}
 		}
 		break;
 		// Get messages
@@ -390,29 +488,9 @@ int main() {
 			}
 
 			// Print out the received messages
-			for(auto msg : msgs) {
-				switch(msg->network.getType()) {
-				case icsneo::Network::Type::CAN:
-				{
-					// A message of type CAN is guaranteed to be a CANMessage, so we can static cast safely
-					auto canMsg = std::static_pointer_cast<icsneo::CANMessage>(msg);
-					std::cout << "\t0x" << std::setfill('0') << std::setw(3) << std::hex << (int) canMsg->arbid << " [" << canMsg->data.size() << "] " << std::dec;
-					
-					for(auto data : canMsg->data) {
-						std::cout << std::setfill('0') << std::setw(2) << std::hex << (int) data << " " << std::dec;
-					}
+			for(const auto& msg : msgs)
+				printMessage(msg);
 
-					std::cout << canMsg->timestamp << std::endl;
-					break;
-				}
-				default:
-					if(msg->network.getNetID() != icsneo::Network::NetID::Device) {
-						std::cout << "\tMessage on netid " << msg->network.GetNetIDString(msg->network.getNetID()) << " with length " << msg->data.size() << std::endl;
-					}
-					break;
-				}
-			}
-						
 			std::cout << std::endl;
 		}
 
@@ -430,7 +508,7 @@ int main() {
 
 			std::cout << "Transmitting a normal CAN frame..." << std::endl;
 			auto msg = std::make_shared<icsneo::CANMessage>();
-			msg->network = icsneo::Network::NetID::HSCAN;
+			msg->network = icsneo::Network::NetID::DWCAN_01;
 			msg->arbid = 0x120;
 			msg->data.insert(msg->data.end(), {0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff});
 			msg->isExtended = false;
@@ -443,11 +521,11 @@ int main() {
 				std::cout << "Failed to transmit message to " << selectedDevice->describe() << "!" << std::endl << std::endl;
 				std::cout << icsneo::GetLastError() << std::endl;;
 			}
-			
+
 			/** Example of CAN FD
 			std::cout << "Transmitting an extended CAN FD frame... " << std::endl;
 			auto txMessage = std::make_shared<icsneo::CANMessage>();
-			txMessage->network = icsneo::Network::NetID::HSCAN;
+			txMessage->network = icsneo::Network::NetID::DWCAN_01;
 			txMessage->arbid = 0x1C5001C5;
 			txMessage->data.insert(txMessage->data.end(), {0xaa, 0xbb, 0xcc});
 			// The DLC will come from the length of the data vector
@@ -487,7 +565,7 @@ int main() {
 			selectedDevice = selectDevice(devices);
 
 			// Attempt to set baudrate and apply settings
-			if(selectedDevice->settings->setBaudrateFor(icsneo::Network::NetID::HSCAN, 250000) && selectedDevice->settings->apply()) {
+			if(selectedDevice->settings->setBaudrateFor(icsneo::Network::NetID::DWCAN_01, 250000) && selectedDevice->settings->apply()) {
 				std::cout << "Successfully set HS CAN baudrate for " << selectedDevice->describe() << " to 250k!" << std::endl;
 			} else {
 				std::cout << "Failed to set HS CAN baudrate for " << selectedDevice->describe() << " to 250k!" << std::endl << std::endl;
@@ -508,7 +586,7 @@ int main() {
 			selectedDevice = selectDevice(devices);
 
 			// Attempt to set baudrate and apply settings
-			if(selectedDevice->settings->setBaudrateFor(icsneo::Network::NetID::LSFTCAN, 250000) && selectedDevice->settings->apply()) {
+			if(selectedDevice->settings->setBaudrateFor(icsneo::Network::NetID::LSFTCAN_01, 250000) && selectedDevice->settings->apply()) {
 				std::cout << "Successfully set LSFT CAN baudrate for " << selectedDevice->describe() << " to 250k!" << std::endl;
 			} else {
 				std::cout << "Failed to set LSFT CAN baudrate for " << selectedDevice->describe() << " to 250k!" << std::endl << std::endl;
@@ -536,28 +614,8 @@ int main() {
 			switch(selection) {
 			case '1':
 			{
-				// Shameless copy-paste from get messages above, demonstrating a callback
-				int callbackID = selectedDevice->addMessageCallback(icsneo::MessageCallback([](std::shared_ptr<icsneo::Message> msg){ 
-					switch(msg->network.getType()) {
-					case icsneo::Network::Type::CAN:
-					{
-						// A message of type CAN is guaranteed to be a CANMessage, so we can static cast safely
-						auto canMsg = std::static_pointer_cast<icsneo::CANMessage>(msg);
-						std::cout << "\t0x" << std::setfill('0') << std::setw(3) << std::hex << (int) canMsg->arbid << " [" << canMsg->data.size() << "] " << std::dec;
-						
-						for(auto data : canMsg->data) {
-							std::cout << std::setfill('0') << std::setw(2) << std::hex << (int) data << " " << std::dec;
-						}
-
-						std::cout << canMsg->timestamp << std::endl;
-						break;
-					}
-					default:
-						if(msg->network.getNetID() != icsneo::Network::NetID::Device) {
-							std::cout << "\tMessage on netid " << msg->network.GetNetIDString(msg->network.getNetID()) << " with length " << msg->data.size() << std::endl;
-						}
-						break;
-					}
+				int callbackID = selectedDevice->addMessageCallback(std::make_shared<icsneo::MessageCallback>([](std::shared_ptr<icsneo::Message> msg){
+					printMessage(msg);
 				}));
 
 				if(callbackID != -1) {
@@ -576,7 +634,7 @@ int main() {
 					break;
 				} else {
 					std::vector<char> allowed;
-					
+
 					std::cout << "Which id would you like to remove?" << std::endl;
 					for(int id : callbacks.find(selectedDevice)->second) {
 						allowed.push_back(static_cast<char>(id) + '0');
@@ -593,7 +651,7 @@ int main() {
 					} else {
 						std::cout << "Failed to remove message callback id " << removeID << " from " << selectedDevice->describe() << "!" << std::endl << std::endl;
 						std::cout << icsneo::GetLastError() << std::endl;;
-					}					
+					}
 				}
 			}
 			break;
@@ -603,6 +661,161 @@ int main() {
 			}
 		}
 		break;
+		// Set Digital IO
+		case 'L':
+		case 'l':
+		{
+			// Select a device and get its description
+			if(devices.size() == 0) {
+				std::cout << "No devices found! Please scan for new devices." << std::endl << std::endl;
+				break;
+			}
+			selectedDevice = selectDevice(devices);
+
+			const auto ethAct = selectedDevice->getEthernetActivationLineCount();
+			const auto usbHost = selectedDevice->getUSBHostPowerCount();
+			const auto backup = selectedDevice->getBackupPowerSupported();
+
+			if(ethAct == 0 && usbHost == 0 && !backup) {
+				std::cout << "The selected device has no controllable digital IO." << std::endl << std::endl;
+				break;
+			}
+
+			std::vector<char> options = { '1' };
+			std::vector<std::string> names;
+			std::map< char, std::pair<icsneo::IO, size_t> > types;
+
+			std::cout << "Select from the following:" << std::endl;
+			for(size_t i = 1; i <= ethAct; i++) {
+				if(i > 1)
+					options.push_back(options.back() + 1);
+				names.push_back("Ethernet (DoIP) Activation Line");
+				if(ethAct > 1) {
+					names.back() += ' ';
+					names.back() += std::to_string(i);
+				}
+				std::cout << '[' << options.back() << "] " << names.back();
+				const auto val = selectedDevice->getDigitalIO(icsneo::IO::EthernetActivation, i);
+				types[options.back()] = { icsneo::IO::EthernetActivation, i };
+				if(val) {
+					if(*val)
+						std::cout << ": Enabled" << std::endl;
+					else
+						std::cout << ": Disabled" << std::endl;
+				} else {
+					std::cout << ": Unknown (" << icsneo::GetLastError() << ")" << std::endl;
+				}
+			}
+			for(size_t i = 1; i <= usbHost; i++) {
+				options.push_back(options.back() + 1);
+				names.push_back("USB Host Power");
+				if(usbHost > 1) {
+					names.back() += ' ';
+					names.back() += std::to_string(i);
+				}
+				std::cout << '[' << options.back() << "] " << names.back();
+				const auto val = selectedDevice->getDigitalIO(icsneo::IO::USBHostPower, i);
+				types[options.back()] = { icsneo::IO::USBHostPower, i };
+				if(val) {
+					if(*val)
+						std::cout << ": Enabled" << std::endl;
+					else
+						std::cout << ": Disabled" << std::endl;
+				} else {
+					std::cout << ": Unknown (" << icsneo::GetLastError() << ")" << std::endl;
+				}
+			}
+			if(backup) {
+				options.push_back(options.back() + 1);
+				names.push_back("Backup Power");
+				std::cout << '[' << options.back() << "] " << names.back() << ": ";
+				auto val = selectedDevice->getDigitalIO(icsneo::IO::BackupPowerEnabled);
+				types[options.back()] = { icsneo::IO::BackupPowerEnabled, 1 };
+				if(val) {
+					if(*val)
+						std::cout << "Enabled";
+					else
+						std::cout << "Disabled";
+
+					val = selectedDevice->getDigitalIO(icsneo::IO::BackupPowerGood);
+					if(val) {
+						if(*val)
+							std::cout << " and Charged" << std::endl;
+						else
+							std::cout << " and Not Charged" << std::endl;
+					} else {
+						std::cout << "but the status is unknown (" << icsneo::GetLastError() << ")" << std::endl;
+					}
+				} else {
+					std::cout << "Unknown (" << icsneo::GetLastError() << ")" << std::endl;
+				}
+			}
+
+			options.push_back(options.back() + 1);
+			std::cout << '[' << options.back() << "] Cancel" << std::endl << std::endl;
+
+			char selection = getCharInput(options);
+			std::cout << std::endl;
+
+			if(selection == options.back()) {
+				std::cout << "Canceling!" << std::endl << std::endl;
+				break;
+			}
+
+			std::cout << "[0] Disable\n[1] Enable\n[2] Cancel" << std::endl << std::endl;
+			char selection2 = getCharInput({ '0', '1', '2' });
+			std::cout << std::endl;
+
+			if(selection2 == '2') {
+				std::cout << "Canceling!" << std::endl << std::endl;
+				break;
+			}
+
+			const bool val = selection2 == '1';
+			if(selectedDevice->setDigitalIO(types[selection].first, types[selection].second, val))
+				std::cout << "OK!" << std::endl << std::endl;
+			else
+				std::cout << "Failure! (" << icsneo::GetLastError() << ")" << std::endl << std::endl;
+		}
+		break;
+		// Set HS CAN termination
+		case 'M':
+		case 'm':
+		{
+			// Select a device and get its description
+			if(devices.size() == 0) {
+				std::cout << "No devices found! Please scan for new devices." << std::endl << std::endl;
+				break;
+			}
+			selectedDevice = selectDevice(devices);
+
+			std::cout << "Termination is ";
+			const auto val = selectedDevice->settings->isTerminationEnabledFor(icsneo::Network::NetID::DWCAN_01);
+			if(!val.has_value()) {
+				std::cout << "not available at this time: " << icsneo::GetLastError() << std::endl << std::endl;
+				break;
+			}
+			std::cout << (*val ? "currently enabled" : "currently disabled") << std::endl;
+
+			std::cout << "[0] Disable\n[1] Enable\n[2] Cancel" << std::endl << std::endl;
+			char selection2 = getCharInput({ '0', '1', '2' });
+			std::cout << std::endl;
+
+			if(selection2 == '2') {
+				std::cout << "Canceling!" << std::endl << std::endl;
+				break;
+			}
+
+			// Attempt to set termination and apply settings
+			if(selectedDevice->settings->setTerminationFor(icsneo::Network::NetID::DWCAN_01, selection2 == '1') && selectedDevice->settings->apply()) {
+				std::cout << "Successfully set HS CAN termination for " << selectedDevice->describe() << std::endl;
+			} else {
+				std::cout << "Failed to set HS CAN termination for " << selectedDevice->describe() << std::endl;
+				std::cout << icsneo::GetLastError() << std::endl;;
+			}
+			std::cout << std::endl;
+		}
+		break;
 		// Exit
 		case 'X':
 		case 'x':
