@@ -1,6 +1,8 @@
 #ifndef __CANMESSAGEFILTER_H_
 #define __CANMESSAGEFILTER_H_
 
+#ifdef __cplusplus
+
 #include "icsneo/communication/message/filter/messagefilter.h"
 #include "icsneo/communication/network.h"
 #include "icsneo/communication/message/message.h"
@@ -11,8 +13,8 @@ namespace icsneo {
 
 class CANMessageFilter : public MessageFilter {
 public:
-	CANMessageFilter() : MessageFilter(Network::Type::CAN), arbid(INVALID_ARBID) {}
-	CANMessageFilter(uint32_t arbid) : MessageFilter(Network::Type::CAN), arbid(arbid) {}
+	CANMessageFilter() : MessageFilter(Network::Type::CAN), arbid(INVALID_ARBID) { messageType = Message::Type::Frame; }
+	CANMessageFilter(uint32_t arbid) : MessageFilter(Network::Type::CAN), arbid(arbid) { messageType = Message::Type::Frame; }
 
 	bool match(const std::shared_ptr<Message>& message) const {
 		if(!MessageFilter::match(message))
@@ -33,6 +35,8 @@ private:
 	}
 };
 
-};
+}
+
+#endif // __cplusplus
 
 #endif
\ No newline at end of file
